﻿using System;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Interop;

namespace TestProject;

/// <summary>
/// 메인 윈도우
/// </summary>
public partial class MainWindow : Window
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Structure
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region 사각형 - RECT

    /// <summary>
    /// 사각형
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    private struct RECT
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 왼쪽
        /// </summary>
        public int Left;

        /// <summary>
        /// 위쭉
        /// </summary>
        public int Top;

        /// <summary>
        /// 오른쪽
        /// </summary>
        public int Right;

        /// <summary>
        /// 아래쪽
        /// </summary>
        public int Bottom;

        #endregion
    }

    #endregion
    #region 모니터 정보 - MONITOR_INFORMATION

    /// <summary>
    /// 모니터 정보
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    private struct MONITOR_INFORMATION
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 크기
        /// </summary>
        public uint Size;

        /// <summary>
        /// 모니터 사각형
        /// </summary>
        public RECT MonitorRectangle;

        /// <summary>
        /// 작업 영역 사각형
        /// </summary>
        public RECT WorkAreaRectangle;

        /// <summary>
        /// 플래그
        /// </summary>
        public uint Flag;

        #endregion
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Import
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 시스템 메트릭스 구하기 - GetSystemMetrics(index)

    /// <summary>
    /// 시스템 메트릭스 구하기
    /// </summary>
    /// <param name="index">인덱스</param>
    /// <returns>처리 결과</returns>
    [DllImport("user32")]
    private static extern int GetSystemMetrics(int index);

    #endregion
    #region 윈도우에서 모니터 구하기 - MonitorFromWindow(windowHandle, flag)

    /// <summary>
    /// 윈도우에서 모니터 구하기
    /// </summary>
    /// <param name="windowHandle">윈도우 핸들</param>
    /// <param name="flag">플래그</param>
    /// <returns>모니터 핸들</returns>
    [DllImport("user32")]
    private static extern IntPtr MonitorFromWindow(IntPtr windowHandle, uint flag);

    #endregion
    #region 모니터 정보 구하기 - GetMonitorInfo(monitorHandle, monitorInformation)

    /// <summary>
    /// 모니터 정보 구하기
    /// </summary>
    /// <param name="monitorHandle">모니터 핸들</param>
    /// <param name="monitorInformation">모니터 정보</param>
    /// <returns>처리 결과</returns>
    [DllImport("user32")]
    private static extern bool GetMonitorInfo(IntPtr monitorHandle, ref MONITOR_INFORMATION monitorInformation);

    #endregion
    #region 시스템 매개 변수 정보 설정하기 - SystemParametersInfo(action, parameter1, parameter2, flag)

    /// <summary>
    /// 시스템 매개 변수 정보 설정하기
    /// </summary>
    /// <param name="action">액션</param>
    /// <param name="parameter1">매개 변수 1</param>
    /// <param name="parameter2">매개 변수 2</param>
    /// <param name="flag">플래그</param>
    /// <returns>처리 결과</returns>
    [DllImport("user32", SetLastError = true)]
    private static extern bool SystemParametersInfo(uint action, uint parameter1, IntPtr parameter2, uint flag);

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// SM_CXSCREEN
    /// </summary>
    private const int SM_CXSCREEN = 0;

    /// <summary>
    /// SM_CYSCREEN
    /// </summary>
    private const int SM_CYSCREEN = 1;

    /// <summary>
    /// MONITOR_DEFAULTTONEAREST
    /// </summary>
    private const uint MONITOR_DEFAULTTONEAREST = 2;

    /// <summary>
    /// SPI_SETWORKAREA
    /// </summary>
    private const uint SPI_SETWORKAREA = 0x002F;

    /// <summary>
    /// 원본 작업 영역 사각형
    /// </summary>
    private RECT originalWorkAreaRectangle;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - MainWindow()

    /// <summary>
    /// 생성자
    /// </summary>
    public MainWindow()
    {
        InitializeComponent();

        Loaded  += Window_Loaded;
        Closing += Window_Closing;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Private
    //////////////////////////////////////////////////////////////////////////////// Event

    #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

    /// <summary>
    /// 윈도우 로드시 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void Window_Loaded(object sender, RoutedEventArgs e)
    {
        SetWindowPosition();
        SetWorkArea();
        SetTopMostWindow();
    }

    #endregion
    #region 윈도우 닫을 경우 처리하기 - Window_Closing(sender, e)

    /// <summary>
    /// 윈도우 닫을 경우 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void Window_Closing(object sender, CancelEventArgs e)
    {
        RestoreWorkArea();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////// Function

    #region 윈도우 위치 설정하기 - SetWindowPosition()

    /// <summary>
    /// 윈도우 위치 설정하기
    /// </summary>
    private void SetWindowPosition()
    {
        IntPtr windowHandle = new WindowInteropHelper(this).Handle;

        IntPtr monitorHandle = MonitorFromWindow(windowHandle, MONITOR_DEFAULTTONEAREST);

        MONITOR_INFORMATION monitorInformation = new MONITOR_INFORMATION();

        monitorInformation.Size = (uint)Marshal.SizeOf(typeof(MONITOR_INFORMATION));

        GetMonitorInfo(monitorHandle, ref monitorInformation);

        RECT workAreaRectangle = monitorInformation.WorkAreaRectangle;

        Width  = 400 + 7;
        Height = workAreaRectangle.Bottom - workAreaRectangle.Top + 7;
        Left   = workAreaRectangle.Right - 400;
        Top    = workAreaRectangle.Top;
    }

    #endregion
    #region 작업 영역 설정하기 - SetWorkArea()

    /// <summary>
    /// 작업 영역 설정하기
    /// </summary>
    private void SetWorkArea()
    {
        IntPtr windowHandle = new WindowInteropHelper(this).Handle;

        IntPtr monitorHandle = MonitorFromWindow(windowHandle, MONITOR_DEFAULTTONEAREST);

        MONITOR_INFORMATION monitorInformation = new MONITOR_INFORMATION();

        monitorInformation.Size = (uint)Marshal.SizeOf(typeof(MONITOR_INFORMATION));

        GetMonitorInfo(monitorHandle, ref monitorInformation);

        this.originalWorkAreaRectangle = monitorInformation.WorkAreaRectangle;

        RECT workAreaRectangle = monitorInformation.WorkAreaRectangle;

        workAreaRectangle.Right -= (int)Width;

        IntPtr workAreaRectangleHandle = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(RECT)));

        Marshal.StructureToPtr(workAreaRectangle, workAreaRectangleHandle, false);

        SystemParametersInfo(SPI_SETWORKAREA, 0, workAreaRectangleHandle, 0);

        Marshal.FreeHGlobal(workAreaRectangleHandle);
    }

    #endregion
    #region 작업 영역 복원하기 - RestoreWorkArea()

    /// <summary>
    /// 작업 영역 복원하기
    /// </summary>
    private void RestoreWorkArea()
    {
        IntPtr workAreaRectangleHandle = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(RECT)));

        Marshal.StructureToPtr(originalWorkAreaRectangle, workAreaRectangleHandle, false);

        SystemParametersInfo(SPI_SETWORKAREA, 0, workAreaRectangleHandle, 0);

        Marshal.FreeHGlobal(workAreaRectangleHandle);
    }

    #endregion
    #region 최상위 윈도우 설정하기 - SetTopMostWindow()

    /// <summary>
    /// 최상위 윈도우 설정하기
    /// </summary>
    private void SetTopMostWindow()
    {
        Topmost = true;
    }

    #endregion
}